//
//  CalculatorRecord.swift
//  calculator-history
//
//  Created by Илья Лошкарёв on 24.03.2018.
//  Copyright © 2018 SFEDU. All rights reserved.
//

import Foundation
import CoreData

@objc(CalculatorRecord)
/// Запись об одной операции
final class CalculatorRecord: NSManagedObject {
    
    // MARK: - Core Data Entity properties
    
    // данные операции
    @NSManaged var leftValue: Double
    @NSManaged private var operationValue: String
    @NSManaged var rightValue: Double
    @NSManaged private var resultValue: NSNumber?
    // дата создания записи
    @NSManaged private var date: Date // необходима для сортировки
    
    // MARK: - Преобразование хранимых данных
    
    var operation: Operation {
        get { return Operation(rawValue: operationValue)! }
        set { operationValue = newValue.rawValue }
    }
    
    var result: Double? {
        get { return resultValue != nil ? Double(truncating: resultValue!) : nil }
        set { resultValue = newValue != nil ? NSNumber(value: newValue!) : nil }
    }
    
    // MARK: -
    
    override init(entity: NSEntityDescription, insertInto context: NSManagedObjectContext?) {
        super.init(entity: entity, insertInto: context)
    }
    
    convenience init(leftValue: Double, operation: Operation, rightValue: Double, result: Double?) {
        self.init(entity: CalculatorRecord.entity(), insertInto: nil)
        
        self.leftValue = leftValue
        self.operation = operation
        self.rightValue = rightValue
        self.result = result
        self.date = Date()
    }

}

extension CalculatorRecord: Fetchable {
    static func fetchAll() -> NSFetchRequest<CalculatorRecord>?{
        guard let entityName = self.entity().name else { return nil }
        return NSFetchRequest<CalculatorRecord>(entityName: entityName);
    }
}
